use once_cell::sync::OnceCell;
use paris::*;
use scrap::{Capturer, Display};
use tracing::field::display;
use std::{
    borrow::BorrowMut,
    sync::{Arc, Mutex},
};

use crate::screen::Screen;

pub struct ScreenColorPicker {
    pub screens: Arc<Mutex<Vec<Screen>>>,
}

impl ScreenColorPicker {
    pub fn global() -> &'static ScreenColorPicker {
        static SCREEN_COLOR_PICKER: OnceCell<ScreenColorPicker> = OnceCell::new();

        SCREEN_COLOR_PICKER.get_or_init(|| ScreenColorPicker {
            screens: Arc::new(Mutex::new(Vec::<Screen>::new())),
        })
    }

    pub fn refresh_displays(&self) -> anyhow::Result<()> {
        let displays = Display::all()
            .map_err(|error| anyhow::anyhow!("Can not get all of displays. {}", error))?;
        let mut screens = self
            .screens
            .lock()
            .map_err(|error| anyhow::anyhow!("lock screens failed. {}", error))?;
        screens.clear();
        info!("number of displays: {}", displays.len());
        for display in displays {
            let height = display.height();
            let width = display.width();
            match Capturer::new(display) {
                Ok(capturer) => screens.push(Screen::new(capturer, width, height)),
                Err(error) => screens.push(Screen::new_failed(
                    anyhow::anyhow!("{}", error),
                    width,
                    height,
                )),
            }
        }

        screens.reverse();
        screens[0].set_number_of_leds(22, 0);
        screens[1].set_number_of_leds(38, 0);

        Ok(())
    }

    pub fn take_screenshots_for_all(&self) -> anyhow::Result<Vec<Vec<u8>>> {
        let mut screens = self
            .screens
            .lock()
            .map_err(|error| anyhow::anyhow!("lock screens failed. {}", error))?;
        let mut screenshots = Vec::<Vec<u8>>::new();
        for screen in screens.iter_mut() {
            let screenshot = screen.take().map_err(|error| {
                anyhow::anyhow!("take screenshot for display failed. {}", error)
            })?;
            screenshots.push(screenshot);
        }

        anyhow::Ok(screenshots)
    }
}
